import xml.sax

class InningScoresXml(xml.sax.ContentHandler):
    def __init__(self):
        self.hits = []

    def startElement(self, name, attrs):
        if name == 'hip':
            self.hits.append(dict(attrs))


import requests
class MLB2:
    """
    Handles all calls from the new statsapi
    """
    url_base = "https://statsapi.mlb.com/api/v1"
    mlb_sport_id = 1


    def conferences(self ,conferenceId, season):
        url = "{0}/api/v1/conferences?conferenceId={0}&season={1}".format(self.url_base,conferenceId, season)
        resp = requests.get(url)
        print(resp.text)



    def config(self):
        url = "{0}/api/v1/gameStatus".format(self.url_base)
        resp = requests.get(url)
        print(resp.text)


    def divisions(self,divisionId, leagueId, sportId):
        url = "https://statsapi.mlb.com/api/v1/divisions?divisionId=200&leagueId=103&sportId=1"
        resp = requests.get(url)
        print(resp.text)


    def draft(self, year):
        "https: // statsapi.mlb.com / api / v1 / draft / 2018?limit = 1 & round = 1 & name = M & school = A & position = P & teamId = 116 & playerId = 663554 & bisPlayerId = 759143"
        resp = requests.get(url)
        print(resp.text)



    def game(self, gameId, timecode):
        url = "{0}/game/{1}/feed/live?".format(self.url_base, gameId)
        resp = requests.get(url)
        print(resp.text)

    def games(self):
        url = "{0}/schedule/games/tied?season={1}&hydrate=linescore".format(self.url_base, season)

    def homeRunDerby(self):
        url = "{0}".format(self.url_base)
        resp = requests.get(url)
        print(resp.text)


    def league(self):
        url = "{0}/league?leagueIds=103&seasons=2018".format(self.url_base)
        resp = requests.get(url)
        print(resp.text)

    def person(self, personsId):
        url = "{0}/people?personsIds={1}".format(self.url_base, personIds)
        resp = requests.get(url)
        print(resp.text)


    def schedule_games(self, season):
        url = "{0}/schedule/games/tied?season={1}&hydrate=linescore".format(self.url_base, season)

    def schedule(self):
        url = "{0}/schedule?sportId=1".format(self.url_base)
        resp = requests.get(url)
        print(resp.text)

    def seasons(self, sportId=1):
        url = "{0}/seasons?sportId={1}".format(self.url_base, sportId)
        resp = requests.get(url)
        print(resp.text)
        return resp.text

    def situationCodes(self):
        url = "{0}/situationCodes".format(self.url_base)
        resp = requests.get(url)
        print(resp.text)

    def sports(self, sportId=1):
        url = "{0}/sports/{1}".format(self.url_base, sportId)
        resp = requests.get(url)
        print(resp.text)

    def standings(self, leagueId, season):
        url = "{0}/standings?leagueId={1}&season={2}".format(self.url_base, leagueId, season)
        resp = requests.get(url)
        print(resp.text)

    def stats(self):
        url = "{0}/stats?stats=season&group=hitting".format(self.url_base)
        resp = requests.get(url)
        print(resp.text)

    def team(self):
        url = "{0}/leaders?leaderCategories=homeRuns&hydrate=team".format(self.url_base)
        resp = requests.get(url)
        print(resp.text)

    def venue(self):
        url = "{0}/venues?venueIds=15".format(self.url_base)
        resp = requests.get(url)
        print(resp.text)


from datetime import date


class NBA_Season_2014(object):
    def __init__(self):
        self.description    = "2014–15 NBA season"
        self.start_date     = date(year=2014,month=10, day=31)
        self.end_date       = date(year=2015, month=4, day=18)

        self.playoff_start_date = date(year=2015, month=4, day=21)
        self.playoff_end_date   = date(year=2015, month=6, day=15)

        self.finals_start_date = date(year=2015, month=6, day=8)
        self.finals_end_date = date(year=2015, month=6, day=20)


import xml.sax
from sports.models.Boxscore import Boxscore

class BoxscoreXml(xml.sax.ContentHandler):
    """
    Parse the boxscore.xml file
    """
    def __init__(self):
        self.boxscore = Boxscore()

    def startElement(self, name, attrs):
        if name == 'boxscore':
            self.boxscore.info = dict(attrs)
            for k,v in attrs.items():
                setattr(self.boxscore,k,v)
        elif name == 'linescore':
            self.boxscore.linescore = dict(attrs)
            for k,v in attrs.items():
                self.boxscore.linescore[k] = v
        elif name == 'inning_line_score':
            self.boxscore.inning_line_scores.append(dict(attrs))


import logging
import requests
import requests_cache
import json
import xml
from sports.mlb.handlers import BenchXml
from sports.mlb.handlers import InningsAllXml
from sports.mlb.handlers.InningHit import InningHitXml
from sports.mlb.handlers.InningScoresXml import InningScoresXml
from sports.mlb.handlers.BoxscoreXml import BoxscoreXml
from sports.mlb.handlers.ScoreboardXml import ScoreboardXml
from sports.mlb.handlers.GameXml import GameXml
from sports.mlb.handlers.GamedaySynXml import GamedaySynXml
from sports.mlb.handlers import GameEventsXml

class MLB:
    domain_name = 'mlb.com'
    pitch_types = { 'CH':'Changeup',
                    'CU':'Curveball',
                    'FC':'Cutter',
                    'EP':'Eephus',
                    'FO':'Forkball',
                    'FA':'Four-Seam Fastball',
                    'KN':'Knuckleball',
                    'KC':'Knuckle-curve',
                    'SC':'Screwball',
                    'SI':'Sinker',
                    'SL':'Slider',
                    'FS':'Splitter',
                    'FT':'Two-Seam Fastball'
                    }

    def __init__(self, cache_name='sports.mlb', cache_backend='sqlite', cache_expire_after=1209600):
        if cache_name != None:
            requests_cache.install_cache(cache_name, backend=cache_backend, expire_after=cache_expire_after)


    def _getRequest(self, url, no_cache=False):
        """
        Get URL from cache unless it was disabled
        Args:
            url: URL to retrieve
            no_cache: Controls if Cache will be checked

        Returns:
            req

        """
        if (no_cache==False):
            req = requests.get(url)
        else:
            with requests_cache.disabled():
                req = requests.get(url)
        return req

    def benchXml(self, game_id, returnXml = False):
        """
        Retrieves and optionally processes the bench.xml for a given game
        Args:
            game_id: Identifier for the game

        Returns:

        """
        url = "http://gd2.mlb.com/components/game/mlb/year_{0}/month_{1}/day_{2}/gid_{3}/bench.xml"
        year, month, day, _discard = game_id.split('_', 3)
        url = url.format(year, month, day, game_id)
        req = requests.get(url)

        if returnXml == True:
            return req.text

        bench_xml = BenchXml()
        xml.sax.parseString(req.text, bench_xml)
        return bench_xml.bench


    def benchOXml(self, game_id, returnXml = False):
        """
        Retrieves and optionally processes the benchO.xml for a given game
        (The 'official' bench xml file)

        Args:
            game_id: Identifier for the game
            returnXml:

        Returns:

        """
        url = "http://gd2.mlb.com/components/game/mlb/year_{0}/month_{1}/day_{2}/gid_{3}/benchO.xml"
        year, month, day, _discard = game_id.split('_', 3)
        url = url.format(year, month, day, game_id)
        req = requests.get(url)

        if returnXml == True:
            return req.text

        bench_xml = BenchXml()
        xml.sax.parseString(req.text, bench_xml)
        return bench_xml.bench

    def boxscoreXml(self, game_id, returnXml = False):
        """
        Retrieves, and optionally processes the boxscore.xml file

        Args:
            game_id: Identifier for the game
        :return:
        """
        url = "http://gd2.mlb.com/components/game/mlb/year_{0}/month_{1}/day_{2}/gid_{3}/boxscore.xml"
        year, month, day, _discard = game_id.split('_', 3)
        url = url.format(year, month, day, game_id)
        req = requests.get(url)
        print(url)

        if returnXml == True:
            return req.text

        boxscore_xml = BoxscoreXml()
        xml.sax.parseString(req.text,boxscore_xml)
        return boxscore_xml.boxscore

    def careerHittingJson(self,player_id,game_type,returnJson=False):
        """

        Args:
            player_id:
            game_type:
            returnJson:

        Returns:

        """
        url = "http://lookup-service-prod.mlb.com/json/named.sport_career_hitting.bam?league_list_id='mlb'&game_type={0}&player_id={1}"
        url = url.format(game_type, player_id)
        req = requests.get(url)
        json_data = str(req.text, 'ISO-8859-1').encode('utf8')

        if returnJson:
            return json_data

        data = json.loads(json_data)
        return data

    def careerHittingLeagueJson(self,player_id,game_type,returnJson=False):
        """

        Args:
            player_id:
            game_type:
            returnJson:

        Returns:

        """
        url = "http://lookup-service-prod.mlb.com/json/named.sport_career_hitting_lg.bam?league_list_id='mlb'&game_type={0}&player_id={1}"
        url = url.format(game_type, player_id)
        req = requests.get(url)
        json_data = str(req.text, 'ISO-8859-1').encode('utf8')

        if returnJson:
            return json_data

        data = json.loads(json_data)
        return data

    def careerPitchingJson(self,player_id,game_type,returnJson=False):
        """

        Args:
            player_id:
            game_type:
            returnJson:

        Returns:

        """
        url = "http://lookup-service-prod.mlb.com/json/named.sport_career_pitching.bam?league_list_id='mlb'&game_type={0}&player_id={1}"
        url = url.format(game_type, player_id)
        req = requests.get(url)
        json_data = str(req.text, 'ISO-8859-1').encode('utf8')

        if returnJson:
            return json_data

        data = json.loads(json_data)
        return data

    def careerPitchingLeagueJson(self,player_id,game_type,returnJson=False):
        """

        Args:
            player_id:
            game_type:
            returnJson:

        Returns:

        """
        url = "http://lookup-service-prod.mlb.com/json/named.sport_career_pitching_lg.bam?league_list_id='mlb'&game_type={0}&player_id={1}"
        url = url.format(game_type, player_id)
        req = requests.get(url)
        json_data = str(req.text, 'ISO-8859-1').encode('utf8')

        if returnJson:
            return json_data

        data = json.loads(json_data)
        return data

    def copyrightTxt(self):
        """
        Retrieves the MLBAM License governing the usage of their data
        :return: (String) Copyright Text
        """
        url = "http://gdx.mlb.com/components/copyright.txt"
        req = requests.get(url)
        return req.text

    def gameXml(self, game_id, returnXml = False):
        """
        Retrieves, and optionally processes the game.xml file

        Args:
            game_id: Identifier for the game
        Returns:
        """
        url = "http://gd2.mlb.com/components/game/mlb/year_{0}/month_{1}/day_{2}/gid_{3}/game.xml"
        year, month, day, _discard = game_id.split('_', 3)
        url = url.format(year, month, day, game_id)
        req = requests.get(url)

        if returnXml == True:
            return req.text

        game_xml = GameXml()
        xml.sax.parseString(req.text, game_xml)
        return game_xml.game

    def gameday_SynXml(self, game_id, returnXml = False):
        """

        :param game_id:
        :return:
        """
        url = "http://gd2.mlb.com/components/game/mlb/year_{0}/month_{1}/day_{2}/gid_{3}/gameday_Syn.xml"
        year, month, day, _discard = game_id.split('_', 3)
        url = url.format(year, month, day, game_id)
        req = requests.get(url)
        print(url)

        if returnXml == True:
            return req.text

        gameday_syn_xml = GamedaySynXml()
        xml.sax.parseString(req.text,gameday_syn_xml)


    def gameEventsXml(self, game_id, returnXml = False, no_cache=False):
        """
        Retrieve, and optionally process, the game_events.xml file

        Args:
            game_id: Identifier for the game

        Returns:
        """
        url = "http://gd2.mlb.com/components/game/mlb/year_{0}/month_{1}/day_{2}/gid_{3}/game_events.xml"
        year, month, day, _discard = game_id.split('_', 3)
        url = url.format(year, month, day, game_id)
        req = self._getRequest(url, no_cache)
        #print(url)
        if returnXml == True:
            return req.text

        game_events_xml = GameEventsXml()
        xml.sax.parseString(req.text,game_events_xml)
        game_events_xml.game.game_id = game_id
        return game_events_xml.game


    def inningsAllXml(self, game_id, returnXml = False):
        """
        Retrieves, and optionally parses the inning_all.xml

        Args:
            game_id:  MLB game id
            returnXml: Controls if the xml data

        Returns:
            inning_all.xml's content or a Game object
        """

        url = "http://gd2.mlb.com/components/game/mlb/year_{0}/month_{1}/day_{2}/gid_{3}/inning/inning_all.xml"
        year, month, day, _discard = game_id.split('_', 3)
        url = url.format(year, month, day, game_id)
        req = requests.get(url)
        print(url)

        if returnXml == True:
            return req.text

        innings_all_xml = InningsAllXml()
        xml.sax.parseString(req.text, innings_all_xml)
        return innings_all_xml.game

    def inningHitXml(self, game_id, returnXml = False):
        """

        :param game_id:
        :return:
        """
        url = "http://gd2.mlb.com/components/game/mlb/year_{0}/month_{1}/day_{2}/gid_{3}/inning/inning_hit.xml"
        year, month, day, _discard = game_id.split('_', 3)
        url = url.format(year, month, day, game_id)
        req = requests.get(url)

        if returnXml == True:
            return req.text

        inning_hit_xml = InningHitXml()
        xml.sax.parseString(req.text, inning_hit_xml)
        return inning_hit_xml.hits

    def inningScoresXml(self, game_id, returnXml=False):
        """

        :param:
            game_id

        :return:
        @todo Correctly Parse this xml file
        """
        url = "http://gd2.mlb.com/components/game/mlb/year_{0}/month_{1}/day_{2}/gid_{3}/inning/inning_Scores.xml"
        year, month, day, _discard = game_id.split('_', 3)
        url = url.format(year, month, day, game_id)
        req = requests.get(url)

        if returnXml:
            return req.text

        inning_scores_xml = InningScoresXml()
        xml.sax.parseString(req.text,inning_scores_xml)
        return inning_scores_xml.hits


    def playerInfoJson(self, player_id, returnJson=False):
        """
        Retrieves, and optionally process, the player_info.bam JSON endpoint

        Args:
            player_id:

        Returns:

        """
        url = "http://lookup-service-prod.mlb.com/json/named.player_info.bam?sport_code='mlb'&player_id='{0}'"
        url = url.format(player_id)
        req = requests.get(url)

        if (returnJson==True):
            return req.text

        data = json.loads(req.text)
        return data['player_info']['queryResults']['row']

    def playerTeams(self, season=None, player_id=None):
        """

        Args:
            player_id: Player ID (Required)
            season:
        Returns:

        """
        url = "http://lookup-service-prod.mlb.com/json/named.player_teams.bam?season='{0}'&player_id='{1}'"

        url = url.format(season, player_id)

    def projectedHittingJson(self,player_id,season=None,returnJson=False):
        """

        Args:
            player_id:
            season:
            returnJson:

        Returns:

        """
        url = "http://lookup-service-prod.mlb.com/json/named.proj_pecota_hitting.bam?player_id={0}".format(player_id)

        if season!=None:
            url += "&season={0}".format(season)

        req = requests.get(url)
        json_data = str(req.text, 'ISO-8859-1').encode('utf8')

        if returnJson:
            return json_data

        data = json.loads(json_data)
        return data

    def projectedPitchingJson(self,player_id,season=None,returnJson=False):
        """

        Args:
            player_id:
            season:
            returnJson:

        Returns:

        """
        url = "http://lookup-service-prod.mlb.com/json/named.proj_pecota_pitching.bam?player_id={0}".format(player_id)

        if season!=None:
            url += "&season={0}".format(season)

        req = requests.get(url)
        json_data = str(req.text, 'ISO-8859-1').encode('utf8')

        if returnJson:
            return json_data

        data = json.loads(json_data)
        return data

    def rawboxscoreXml(self, game_id, returnXml = False):
        """

        :param game_id:
        :return:
        """
        url = "http://gd2.mlb.com/components/game/mlb/year_{0}/month_{1}/day_{2}/gid_{3}/rawboxscore.xml"
        year, month, day, _discard = game_id.split('_', 3)
        url = url.format(year, month, day, game_id)
        req = requests.get(url)

        if returnXml:
            return req.text

        boxscore_xml = BoxscoreXml()
        xml.sax.parseString(req.text, boxscore_xml)
        return boxscore_xml.boxscore


    def scoreboardXml(self, lookup_date, returnXml=False, no_cache=False):
        """
        Retrieves, and optionally processes the scoreboard.xml file for a given date.
        :param:
            lookup_date

        :returns:
            Scoreboard, if returnXml == False
            (XML) string, if returnXml == True
        """
        url = "http://gd2.mlb.com/components/game/mlb/year_{0}/month_{1:02d}/day_{2:02d}/scoreboard.xml"
        url = url.format(lookup_date.year, lookup_date.month, lookup_date.day)
        req = self._getRequest(url,no_cache)
        logging.debug("scoreboardXml URL: {0}".format(url))

        if returnXml:
            return req.text

        scoreboardXml = ScoreboardXml()
        xml.sax.parseString(req.text, scoreboardXml)
        return scoreboardXml.scoreboard


    def searchPlayerAllJson(self,active_sw=None,name_part=None,returnJson=False):
        """

        Args:
            active_sw:
            name_part:

        Returns:

        """
        url = "http://lookup-service-prod.mlb.com/json/named.search_player_all.bam?sport_code = 'mlb'"


        if (active_sw):
            url += "&active_sw={0}".format(active_sw)

        if (name_part):
            url += "&name_part={0}".format(name_part)

        req = requests.get(url)
        json_data = str(req.text, 'ISO-8859-1').encode('utf8')

        if returnJson:
            return json_data

        data = json.loads(json_data)
        return data


    def sportHittingJson(self,player_id, game_type, season, returnJson=False):
        """

        Args:
            player_id:
            game_type:
            season:
            returnJson:

        Returns:

        """
        url = "http://lookup-service-prod.mlb.com/json/named.sport_hitting_tm.bam?league_list_id='mlb'&game_type={0}&season={1}&player_id={2}"
        url = url.format(game_type,season,player_id)
        req = requests.get(url)
        json_data = str(req.text, 'ISO-8859-1').encode('utf8')

        if returnJson:
            return json_data

        data = json.loads(json_data)
        return data

    def sportPitchingJson(self, player_id, game_type, season, returnJson=False):
        """
        Args:
            game_type:
            season:
            player_id
        Returns:

        """
        url = "http://lookup-service-prod.mlb.com/json/named.sport_pitching_tm.bam?league_list_id='mlb'&game_type={0}&season={1}&player_id={2}"
        url = url.format(game_type,season,player_id)
        req = requests.get(url)
        json_data = str(req.text, 'ISO-8859-1').encode('utf8')

        if returnJson:
           return json_data

        data = json.loads(json_data)
        return data




class NbaTeam:
    """
    An object containing all of a team's season information.

    Finds and parses all team stat information and identifiers, such as rank,
    name, and abbreviation, and sets them as properties which can be directly
    read from for easy reference.

    If calling directly, the team's abbreviation needs to be passed. Otherwise,
    the Teams class will handle all arguments.

    Parameters
    ----------
    team_name : string (optional)
        The name of the team to pull if being called directly.
    team_data : string (optional)
        A string containing all of the rows of stats for a given team. If
        multiple tables are being referenced, this will be comprised of
        multiple rows in a single string. Is only used when called directly
        from the Teams class.
    rank : int (optional)
        A team's position in the league based on the number of points they
        obtained during the season. Is only used when called directly from the
        Teams class.
    year : string (optional)
        The requested year to pull stats from.
    season_file : string (optional)
        Optionally specify the filename of a local file to use to pull data
        instead of downloading from sports-reference.com. This file should be
        of the Season page for the designated year.
    """
    def __init__(self, team_name=None, team_data=None, rank=None, year=None,
                 season_file=None):
        self._year = year
        self._rank = rank
        self._abbreviation = None
        self._name = None
        self._games_played = None
        self._minutes_played = None
        self._field_goals = None
        self._field_goal_attempts = None
        self._field_goal_percentage = None
        self._three_point_field_goals = None
        self._three_point_field_goal_attempts = None
        self._three_point_field_goal_percentage = None
        self._two_point_field_goals = None
        self._two_point_field_goal_attempts = None
        self._two_point_field_goal_percentage = None
        self._free_throws = None
        self._free_throw_attempts = None
        self._free_throw_percentage = None
        self._offensive_rebounds = None
        self._defensive_rebounds = None
        self._total_rebounds = None
        self._assists = None
        self._steals = None
        self._blocks = None
        self._turnovers = None
        self._personal_fouls = None
        self._points = None
        self._opp_field_goals = None
        self._opp_field_goal_attempts = None
        self._opp_field_goal_percentage = None
        self._opp_three_point_field_goals = None
        self._opp_three_point_field_goal_attempts = None
        self._opp_three_point_field_goal_percentage = None
        self._opp_two_point_field_goals = None
        self._opp_two_point_field_goal_attempts = None
        self._opp_two_point_field_goal_percentage = None
        self._opp_free_throws = None
        self._opp_free_throw_attempts = None
        self._opp_free_throw_percentage = None
        self._opp_offensive_rebounds = None
        self._opp_defensive_rebounds = None
        self._opp_total_rebounds = None
        self._opp_assists = None
        self._opp_steals = None
        self._opp_blocks = None
        self._opp_turnovers = None
        self._opp_personal_fouls = None
        self._opp_points = None

        if team_name:
            team_data = self._retrieve_team_data(year, team_name, season_file)
        self._parse_team_data(team_data)

    def __str__(self):
        """
        Return the string representation of the class.
        """
        return f'{self.name} ({self.abbreviation}) - {self._year}'

    def __repr__(self):
        """
        Return the string representation of the class.
        """
        return self.__str__()

    def _retrieve_team_data(self, year, team_name, season_file=None):
        """
        Pull all stats for a specific team.

        By first retrieving a dictionary containing all information for all
        teams in the league, only select the desired team for a specific year
        and return only their relevant results.

        Parameters
        ----------
        year : string
            A ``string`` of the requested year to pull stats from.
        team_name : string
            A ``string`` of the team's 3-letter abbreviation, such as 'HOU' for
            the Houston Rockets.
       season_file : string (optional)
            Optionally specify the filename of a local file to use to pull data
            instead of downloading from sports-reference.com. This file should
            be of the Season page for the designated year.

        Returns
        -------
        PyQuery object
            Returns a PyQuery object containing all stats and information for
            the specified team.
        """
        team_data_dict, year = _retrieve_all_teams(year, season_file)
        self._year = year
        team_data = team_data_dict[team_name]['data']
        self._rank = team_data_dict[team_name]['rank']
        return team_data

    def _parse_team_data(self, team_data):
        """
        Parses a value for every attribute.

        This function looks through every attribute with the exception of
        '_rank' and retrieves the value according to the parsing scheme and
        index of the attribute from the passed HTML data. Once the value is
        retrieved, the attribute's value is updated with the returned result.

        Note that this method is called directly once Team is invoked and does
        not need to be called manually.

        Parameters
        ----------
        team_data : string
            A string containing all of the rows of stats for a given team. If
            multiple tables are being referenced, this will be comprised of
            multiple rows in a single string.
        """
        for field in self.__dict__:
            # The rank attribute is passed directly to the class during
            # instantiation.
            if field == '_rank' or \
               field == '_year':
                continue
            value = utils._parse_field(PARSING_SCHEME,
                                       team_data,
                                       str(field)[1:])
            setattr(self, field, value)

    @property
    def dataframe(self):
        """
        Returns a pandas DataFrame containing all other class properties and
        values. The index for the DataFrame is the string abbreviation of the
        team, such as 'DET'.
        """
        fields_to_include = {
            'abbreviation': self.abbreviation,
            'assists': self.assists,
            'blocks': self.blocks,
            'defensive_rebounds': self.defensive_rebounds,
            'field_goal_attempts': self.field_goal_attempts,
            'field_goal_percentage': self.field_goal_percentage,
            'field_goals': self.field_goals,
            'free_throw_attempts': self.free_throw_attempts,
            'free_throw_percentage': self.free_throw_percentage,
            'free_throws': self.free_throws,
            'games_played': self.games_played,
            'minutes_played': self.minutes_played,
            'name': self.name,
            'offensive_rebounds': self.offensive_rebounds,
            'opp_assists': self.opp_assists,
            'opp_blocks': self.opp_blocks,
            'opp_defensive_rebounds': self.opp_defensive_rebounds,
            'opp_field_goal_attempts': self.opp_field_goal_attempts,
            'opp_field_goal_percentage': self.opp_field_goal_percentage,
            'opp_field_goals': self.opp_field_goals,
            'opp_free_throw_attempts': self.opp_free_throw_attempts,
            'opp_free_throw_percentage': self.opp_free_throw_percentage,
            'opp_free_throws': self.opp_free_throws,
            'opp_offensive_rebounds': self.opp_offensive_rebounds,
            'opp_personal_fouls': self.opp_personal_fouls,
            'opp_points': self.opp_points,
            'opp_steals': self.opp_steals,
            'opp_three_point_field_goal_attempts':
            self.opp_three_point_field_goal_attempts,
            'opp_three_point_field_goal_percentage':
            self.opp_three_point_field_goal_percentage,
            'opp_three_point_field_goals': self.opp_three_point_field_goals,
            'opp_total_rebounds': self.opp_total_rebounds,
            'opp_turnovers': self.opp_turnovers,
            'opp_two_point_field_goal_attempts':
            self.opp_two_point_field_goal_attempts,
            'opp_two_point_field_goal_percentage':
            self.opp_two_point_field_goal_percentage,
            'opp_two_point_field_goals': self.opp_two_point_field_goals,
            'personal_fouls': self.personal_fouls,
            'points': self.points,
            'rank': self.rank,
            'steals': self.steals,
            'three_point_field_goal_attempts':
            self.three_point_field_goal_attempts,
            'three_point_field_goal_percentage':
            self.three_point_field_goal_percentage,
            'three_point_field_goals': self.three_point_field_goals,
            'total_rebounds': self.total_rebounds,
            'turnovers': self.turnovers,
            'two_point_field_goal_attempts':
            self.two_point_field_goal_attempts,
            'two_point_field_goal_percentage':
            self.two_point_field_goal_percentage,
            'two_point_field_goals': self.two_point_field_goals
        }
        return pd.DataFrame([fields_to_include], index=[self._abbreviation])

    @int_property_decorator
    def rank(self):
        """
        Returns an ``int`` of the team's rank based on the number of points
        they score per game.
        """
        return self._rank

    @property
    def abbreviation(self):
        """
        Returns a ``string`` of the team's abbreviation, such as 'DET' for the
        Detroit Pistons.
        """
        return self._abbreviation

    @property
    def schedule(self):
        """
        Returns an instance of the Schedule class containing the team's
        complete schedule for the season.
        """
        return Schedule(self._abbreviation, self._year)

    @property
    def roster(self):
        """
        Returns an instance of the Roster class containing all players for the
        team during the season with all career stats.
        """
        return Roster(self._abbreviation, self._year)

    @property
    def name(self):
        """
        Returns a ``string`` of the team's full name, such as 'Detroit
        Pistons'.
        """
        return self._name

    @int_property_decorator
    def games_played(self):
        """
        Returns an ``int`` of the total number of games the team has played
        during the season.
        """
        return self._games_played

    @int_property_decorator
    def minutes_played(self):
        """
        Returns an ``int`` of the total number of minutes played by all players
        on the team during the season.
        """
        return self._minutes_played

    @int_property_decorator
    def field_goals(self):
        """
        Returns an ``int`` of the total number of field goals the team has made
        during the season.
        """
        return self._field_goals

    @int_property_decorator
    def field_goal_attempts(self):
        """
        Returns an ``int`` of the total number of field goals the team has
        attempted during the season.
        """
        return self._field_goal_attempts

    @float_property_decorator
    def field_goal_percentage(self):
        """
        Returns a ``float`` of the percentage of field goals made divided by
        the number of attempts. Percentage ranges from 0-1.
        """
        return self._field_goal_percentage

    @int_property_decorator
    def three_point_field_goals(self):
        """
        Returns an ``int`` of the total number of three point field goals the
        team has made during the season.
        """
        return self._three_point_field_goals

    @int_property_decorator
    def three_point_field_goal_attempts(self):
        """
        Returns an ``int`` of the total number of three point field goals the
        team has attempted during the season.
        """
        return self._three_point_field_goal_attempts

    @float_property_decorator
    def three_point_field_goal_percentage(self):
        """
        Returns a ``float`` of the percentage of three point field goals made
        divided by the number of attempts. Percentage ranges from 0-1.
        """
        return self._three_point_field_goal_percentage

    @int_property_decorator
    def two_point_field_goals(self):
        """
        Returns an ``int`` of the total number of two point field goals the
        team has made during the season.
        """
        return self._two_point_field_goals

    @int_property_decorator
    def two_point_field_goal_attempts(self):
        """
        Returns an ``int`` of the total number of two point field goals the
        team has attempted during the season.
        """
        return self._two_point_field_goal_attempts

    @float_property_decorator
    def two_point_field_goal_percentage(self):
        """
        Returns a ``float`` of the percentage of two point field goals made
        divided by the number of attempts. Percentage ranges from 0-1.
        """
        return self._two_point_field_goal_percentage

    @int_property_decorator
    def free_throws(self):
        """
        Returns an ``int`` of the total number of free throws made during the
        season.
        """
        return self._free_throws

    @int_property_decorator
    def free_throw_attempts(self):
        """
        Returns an ``int`` of the total number of free throw attempts during
        the season.
        """
        return self._free_throw_attempts

    @float_property_decorator
    def free_throw_percentage(self):
        """
        Returns a ``float`` of the percentage of free throws made divided by
        the attempts. Percentage ranges from 0-1.
        """
        return self._free_throw_percentage

    @int_property_decorator
    def offensive_rebounds(self):
        """
        Returns an ``int`` of the total number of offensive rebounds the team
        has grabbed.
        """
        return self._offensive_rebounds

    @int_property_decorator
    def defensive_rebounds(self):
        """
        Returns an ``int`` of the total number of defensive rebounds the team
        has grabbed.
        """
        return self._defensive_rebounds

    @int_property_decorator
    def total_rebounds(self):
        """
        Returns an ``int`` of the total number of rebounds the team has
        grabbed.
        """
        return self._total_rebounds

    @int_property_decorator
    def assists(self):
        """
        Returns an ``int`` of the total number of field goals that were
        assisted.
        """
        return self._assists

    @int_property_decorator
    def steals(self):
        """
        Returns an ``int`` of the total number of times the team stole the ball
        from the opponent.
        """
        return self._steals

    @int_property_decorator
    def blocks(self):
        """
        Returns an ``int`` of the total number of times the team blocked an
        opponent's shot.
        """
        return self._blocks

    @int_property_decorator
    def turnovers(self):
        """
        Returns an ``int`` of the total number of times the team has turned the
        ball over.
        """
        return self._turnovers

    @int_property_decorator
    def personal_fouls(self):
        """
        Returns an ``int`` of the total number of times the team has fouled an
        opponent.
        """
        return self._personal_fouls

    @int_property_decorator
    def points(self):
        """
        Returns an ``int`` of the total number of points the team has scored
        during the season.
        """
        return self._points

    @int_property_decorator
    def opp_field_goals(self):
        """
        Returns an ``int`` of the total number of field goals the opponents
        made during the season.
        """
        return self._opp_field_goals

    @int_property_decorator
    def opp_field_goal_attempts(self):
        """
        Returns an ``int`` of the total number of field goals the opponents
        attempted during the season.
        """
        return self._opp_field_goal_attempts

    @float_property_decorator
    def opp_field_goal_percentage(self):
        """
        Returns a ``float`` of the percentage of field goals made divided by
        the number of attempts by the opponent. Percentage ranges from 0-1.
        """
        return self._opp_field_goal_percentage

    @int_property_decorator
    def opp_three_point_field_goals(self):
        """
        Returns an ``int`` of the total number of three point field goals the
        opponent made during the season.
        """
        return self._opp_three_point_field_goals

    @int_property_decorator
    def opp_three_point_field_goal_attempts(self):
        """
        Returns an ``int`` of the total number of three point field goals the
        opponent attempted during the season.
        """
        return self._opp_three_point_field_goal_attempts

    @float_property_decorator
    def opp_three_point_field_goal_percentage(self):
        """
        Returns a ``float`` of the percentage of three point field goals made
        divided by the number of attempts by the opponent. Percentage ranges
        from 0-1.
        """
        return self._opp_three_point_field_goal_percentage

    @int_property_decorator
    def opp_two_point_field_goals(self):
        """
        Returns an ``int`` of the total number of two point field goals the
        opponent made during the season.
        """
        return self._opp_two_point_field_goals

    @int_property_decorator
    def opp_two_point_field_goal_attempts(self):
        """
        Returns an ``int`` of the total number of two point field goals the
        opponent attempted during the season.
        """
        return self._opp_two_point_field_goal_attempts

    @float_property_decorator
    def opp_two_point_field_goal_percentage(self):
        """
        Returns a ``float`` of the percentage of two point field goals made
        divided by the number of attempts by the opponent. Percentage ranges
        from 0-1.
        """
        return self._opp_two_point_field_goal_percentage

    @int_property_decorator
    def opp_free_throws(self):
        """
        Returns an ``int`` of the total number of free throws made during the
        season by the opponent.
        """
        return self._opp_free_throws

    @int_property_decorator
    def opp_free_throw_attempts(self):
        """
        Returns an ``int`` of the total number of free throw attempts during
        the season by the opponent.
        """
        return self._opp_free_throw_attempts

    @float_property_decorator
    def opp_free_throw_percentage(self):
        """
        Returns a ``float`` of the percentage of free throws made divided by
        the attempts by the opponent. Percentage ranges from 0-1.
        """
        return self._opp_free_throw_percentage

    @int_property_decorator
    def opp_offensive_rebounds(self):
        """
        Returns an ``int`` of the total number of offensive rebounds the
        opponent grabbed.
        """
        return self._opp_offensive_rebounds

    @int_property_decorator
    def opp_defensive_rebounds(self):
        """
        Returns an ``int`` of the total number of defensive rebounds the
        opponent grabbed.
        """
        return self._opp_defensive_rebounds

    @int_property_decorator
    def opp_total_rebounds(self):
        """
        Returns an ``int`` of the total number of rebounds the opponent
        grabbed.
        """
        return self._opp_total_rebounds

    @int_property_decorator
    def opp_assists(self):
        """
        Returns an ``int`` of the total number of field goals that were
        assisted by the opponent.
        """
        return self._opp_assists

    @int_property_decorator
    def opp_steals(self):
        """
        Returns an ``int`` of the total number of times the opponent stole the
        ball from the team.
        """
        return self._opp_steals

    @int_property_decorator
    def opp_blocks(self):
        """
        Returns an ``int`` of the total number of times the opponent blocked
        the team's shot.
        """
        return self._opp_blocks

    @int_property_decorator
    def opp_turnovers(self):
        """
        Returns an ``int`` of the total number of times the opponent turned the
        ball over.
        """
        return self._opp_turnovers

    @int_property_decorator
    def opp_personal_fouls(self):
        """
        Returns an ``int`` of the total number of times the opponent fouled the
        team.
        """
        return self._opp_personal_fouls

    @int_property_decorator
    def opp_points(self):
        """
        Returns an ``int`` of the total number of points the team has been
        scored on during the season.
        """
        return self._opp_points


# class NbaTeam(object):
#     def __init__(self):
#         self.name = None        # Name of the Team (Without City)
#         self.full_name = None   # Full Name Of The Team (Including City)
#         self.team_id = None



from sports.nba.nba_team import NBA_Team


class NewOrleansPelicans(NBA_Team):
    """
    NBA's New Orleans Pelicans Static Information

    """

    full_name = "New Orleans Pelicans"
    name = "Pelicans"
    team_id = 1610612740

    def __init__(self):
        """
        """
        super().__init__()


import xml.sax
from sports.models.Game import Game
from sports.models.AtBat import AtBat
from sports.models.Bunch import Bunch

class GameEventsXml(xml.sax.ContentHandler):
    def __init__(self):
        self.game = Game()
        self.currentAtBat = None
        self.currentAction = None
        self.currentPitch = None

    def startElement(self, name, attrs):
        if name == 'atbat':
            if self.currentAtBat is not None:
                self.game.at_bats.append(self.currentAtBat)
            self.currentAtBat = AtBat(**attrs)
        elif name == 'atBat':
            pass
        elif name == 'action':
            #For some strange reason you can find actions outside of atbats, game id: 2016_06_19_tormlb_balmlb_1
            if ((self.currentAction is not None) and (self.currentAtBat is not None)):
                self.currentAtBat.actions.append(self.currentAction)
            self.currentAction = Bunch(**attrs)
        elif name == 'bottom':
            pass
        elif name == 'deck':
            pass
        elif name == 'game':
            self.game = Game(**attrs)
        elif name == 'hole':
            pass
        elif name == 'inning':
            pass
        elif name == 'pitch':
            if self.currentPitch is not None:
                self.currentAtBat.pitches.append(self.currentPitch)
            self.currentPitch = Bunch(**attrs)
        elif name == 'top':
            pass
        else:
            print(name)


from sports.nba.nba_team import NBA_Team


class DenverNuggets(NBA_Team):
    full_name   = "Denver Nuggets"
    name        = "Nuggets"
    team_id     = 1610612743

    def __init__(self):
        self.color_schemes   =[['0E2240','FEC524','8B2131','1D428A'],[],[]]
        self.color_years     = [{'start':2018,'end':None},{'start':2003,'end':2017},{'start':1993,'end':2003}]



from datetime import date


class NBA_Season_2004(object):
    def __init__(self):
        self.description    = "2004–05 NBA season"
        self.start_date     = date(year=2004,month=11, day=2)
        self.end_date       = date(year=2005, month=4, day=20)

        self.playoff_start_date = date(year=2005, month=4, day=23)
        self.playoff_end_date   = date(year=2005, month=6, day=6)


from datetime import date


class NBA_Season_2001(object):
    def __init__(self):
        self.description    = "2001–02 NBA season"
        self.start_date     = date(year=2001,month=10, day=30)
        self.end_date       = date(year=2002, month=4, day=17)

        self.playoff_start_date = date(year=2002, month=4, day=20)
        self.playoff_end_date   = date(year=2002, month=6, day=2)